import functools
import logging
import time
from pubhelper.retry_wraps import retry_wraps as r


def timethis(fn=None, *, logger=None):
    """统计代码执行时间"""

    def inner(f):
        @functools.wraps(f)
        def _inner(*args, **kwargs):
            start = time.perf_counter()
            data = f(*args, **kwargs)
            end = time.perf_counter()
            logger.debug(f'func({f.__name__}): {(end - start) * 1000} ms')
            return data

        return _inner

    if not logger:
        logger = logging.getLogger('__time_count__')
    logger.setLevel(logging.DEBUG)
    return callable(fn) and inner(fn) or inner


def with_log(fn=None, *, logger=None, is_debug=False):
    """打印日志"""

    def inner(f):
        @functools.wraps(f)
        def _inner(*args, **kwargs):
            logger.debug(
                f'func({f.__name__}) running(args={args}, kwargs={kwargs})')
            result = r(f, is_debug=is_debug)(*args, **kwargs)
            logger.debug(f'func({f.__name__}) result: {result}')
            return result

        return _inner

    if not logger:
        logger = logging.getLogger('__debug__')
    logger.setLevel(logging.DEBUG)
    return callable(fn) and inner(fn) or inner


__all__ = ('with_log', 'timethis')



import functools
import hashlib
import random
import string
from IPy import intToIp
from typing import Union


def rdm_str(n: int):
    seed = string.digits + string.ascii_letters
    return ''.join(random.choices(seed, k=n))


def md5(data: Union[str, bytes]) -> str:
    if isinstance(data, str):
        data = data.encode(encoding='utf-8')
    return hashlib.md5(data).hexdigest()


def ip_2_int(ip: str):
    # 先把 192.168.1.13 变成16进制的 c0.a8.01.0d ，再去了“.”后转成10进制的 3232235789 即可。
    # (((((192 * 256) + 168) * 256) + 1) * 256) + 13
    return functools.reduce(lambda x, y: (x << 8) + y, map(int, ip.split('.')))


def int_2_ip(ip: int):
    return intToIp(ip, 4)


__all__ = ('rdm_str', 'md5', 'ip_2_int', 'int_2_ip')



import threading
from typing import Any, Optional
from weakref import WeakValueDictionary


class CachedManager(WeakValueDictionary):
    """
    局部变量，新建缓存，当局部变量被销毁时，缓存也会被清除
    """
    thread_lock: threading.Lock = threading.Lock()

    def get(self, key: Any) -> Optional[Any]:
        with self.thread_lock:
            if not super(CachedManager, self).get(key):
                data = CacheItem()
                self.__setitem__(key, data)
            return super(CachedManager, self).get(key)


class CacheItem(dict):
    m: CachedManager = CachedManager()

    def set(self, **kwargs):
        [self.__setitem__(k, v) for k, v in kwargs.items()]


__all__ = ('CacheItem',)



import heapq
from typing import Any, Iterable


class SimplePriorityQueue(list):
    """简单优先队列实现"""

    def __init__(self, *args, **kwargs):
        """禁止从构造函数添加元素"""
        super(SimplePriorityQueue, self).__init__()

    def __getitem__(self, item):
        data = super().__getitem__(item)
        if data and isinstance(data, (tuple, list)):
            return data[-1]
        return data

    def __setitem__(self, key, value):
        super(
            SimplePriorityQueue, self
        ).__setitem__(
            key, (0, key, value)
        )

    def __iter__(self):
        while True:
            yield self.pop_pri()

    def append(self, obj: Any) -> None:
        self.push_pri(obj)

    def extend(self, iterable: Iterable[Any]) -> None:
        [self.append(item) for item in iterable]

    def pop(self, index: int = -1):
        data = super().pop(index)
        if data and isinstance(data, (tuple, list)):
            return data[-1]
        return data

    def pop_pri(self):
        return heapq.heappop(self)[-1] if self else None

    def push_pri(self, item: Any, priority: int = 0):
        heapq.heappush(self, (-priority, len(self), item))


__all__ = ('SimplePriorityQueue', )



import logging

from pubhelper.func import *
from pubhelper.debug import *
from pubhelper.retry_wraps import *
from pubhelper.simple_cache import *
from pubhelper.request_base import *
from pubhelper.simple_priority_queue import *

logging.basicConfig(
    level=logging.INFO,
    format='[%(asctime)s][%(name)s][%(levelname)s] %(message)s'
)
